@@ -1,25 +1,25 @@
 /*****************************************************************************
  * effects.c : Effects for the visualization system
  *****************************************************************************
- * Copyright (C) 2002-2009 the VideoLAN team
- * $Id: 267611ab55a7e137ade406b105f291ccdc2b1f7f $
+ * Copyright (C) 2002-2009 VLC authors and VideoLAN
+ * $Id$
  *
  * Authors: Clément Stenac <zorglub@via.ecp.fr>
  *          Adrien Maglo <magsoft@videolan.org>
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 /*****************************************************************************
@@ -30,13 +30,14 @@
 #endif
 
 #include <vlc_common.h>
-#include <vlc_vout.h>
-#include <vlc_aout.h>
+#include <vlc_picture.h>
+#include <vlc_block.h>
 
 #include "visual.h"
 #include <math.h>
 
 #include "fft.h"
+#include "window.h"
 
 #define PEAK_SPEED 1
 #define BAR_DECREASE_SPEED 5
@@ -48,19 +49,36 @@
 /*****************************************************************************
  * dummy_Run
  *****************************************************************************/
-int dummy_Run( visual_effect_t * p_effect, aout_instance_t *p_aout,
-               aout_buffer_t * p_buffer , picture_t * p_picture)
+static int dummy_Run( visual_effect_t * p_effect, vlc_object_t *p_aout,
+                      const block_t * p_buffer , picture_t * p_picture)
 {
     VLC_UNUSED(p_effect); VLC_UNUSED(p_aout); VLC_UNUSED(p_buffer);
     VLC_UNUSED(p_picture);
     return 0;
 }
 
+static void dummy_Free( void *data )
+{
+    VLC_UNUSED(data);
+}
+
+
 /*****************************************************************************
  * spectrum_Run: spectrum analyser
  *****************************************************************************/
-int spectrum_Run(visual_effect_t * p_effect, aout_instance_t *p_aout,
-                 aout_buffer_t * p_buffer , picture_t * p_picture)
+typedef struct spectrum_data
+{
+    int *peaks;
+    int *prev_heights;
+
+    unsigned i_prev_nb_samples;
+    int16_t *p_prev_s16_buff;
+
+    window_param wind_param;
+} spectrum_data;
+
+static int spectrum_Run(visual_effect_t * p_effect, vlc_object_t *p_aout,
+                        const block_t * p_buffer , picture_t * p_picture)
 {
     spectrum_data *p_data = p_effect->p_data;
     float p_output[FFT_BUFFER_SIZE];  /* Raw FFT Result  */
@@ -87,6 +105,7 @@ int spectrum_Run(visual_effect_t * p_effect, aout_instance_t *p_aout,
     const int *xscale;
 
     fft_state *p_state;                 /* internal FFT data */
+    DEFINE_WIND_CONTEXT( wind_ctx );    /* internal window data */
 
     int i , j , y , k;
     int i_line;
@@ -100,13 +119,44 @@ int spectrum_Run(visual_effect_t * p_effect, aout_instance_t *p_aout,
     int16_t  *p_buffs;                    /* int16_t converted buffer */
     int16_t  *p_s16_buff;                 /* int16_t converted buffer */
 
-    p_s16_buff = malloc( p_buffer->i_nb_samples * p_effect->i_nb_chans * sizeof(int16_t));
-    if( !p_s16_buff )
+    if (!p_buffer->i_nb_samples) {
+        msg_Err(p_aout, "no samples yet");
         return -1;
+    }
 
-    p_buffs = p_s16_buff;
-    i_80_bands = config_GetInt ( p_aout, "visual-80-bands" );
-    i_peak     = config_GetInt ( p_aout, "visual-peaks" );
+    /* Create p_data if needed */
+    if( !p_data )
+    {
+        p_effect->p_data = p_data = malloc( sizeof( spectrum_data ) );
+        if( !p_data )
+            return -1;
+
+        p_data->peaks = calloc( 80, sizeof(int) );
+        p_data->prev_heights = calloc( 80, sizeof(int) );
+
+        p_data->i_prev_nb_samples = 0;
+        p_data->p_prev_s16_buff = NULL;
+
+        window_get_param( p_aout, &p_data->wind_param );
+    }
+    peaks = (int *)p_data->peaks;
+    prev_heights = (int *)p_data->prev_heights;
+
+    /* Allocate the buffer only if the number of samples change */
+    if( p_buffer->i_nb_samples != p_data->i_prev_nb_samples )
+    {
+        free( p_data->p_prev_s16_buff );
+        p_data->p_prev_s16_buff = vlc_alloc( p_buffer->i_nb_samples *
+                                             p_effect->i_nb_chans,
+                                             sizeof(int16_t));
+        p_data->i_prev_nb_samples = p_buffer->i_nb_samples;
+        if( !p_data->p_prev_s16_buff )
+            return -1;
+    }
+    p_buffs = p_s16_buff = p_data->p_prev_s16_buff;
+
+    i_80_bands = var_InheritInteger( p_aout, "visual-80-bands" );
+    i_peak     = var_InheritInteger( p_aout, "visual-peaks" );
 
     if( i_80_bands != 0)
     {
@@ -119,32 +169,9 @@ int spectrum_Run(visual_effect_t * p_effect, aout_instance_t *p_aout,
         i_nb_bands = 20;
     }
 
-    if( !p_data )
-    {
-        p_effect->p_data = p_data = malloc( sizeof( spectrum_data ) );
-        if( !p_data )
-        {
-            free( p_s16_buff );
-            return -1;
-        }
-
-        p_data->peaks = calloc( 80, sizeof(int) );
-        p_data->prev_heights = calloc( 80, sizeof(int) );
-
-        peaks = ( int * )p_data->peaks;
-        prev_heights = ( int * )p_data->prev_heights;
-    }
-    else
-    {
-        peaks = (int *)p_data->peaks;
-        prev_heights = (int *)p_data->prev_heights;
-    }
-
-
-    height = malloc( i_nb_bands * sizeof(int) );
+    height = vlc_alloc( i_nb_bands, sizeof(int) );
     if( !height )
     {
-        free( p_s16_buff );
         return -1;
     }
     /* Convert the buffer to int16_t  */
@@ -163,10 +190,16 @@ int spectrum_Run(visual_effect_t * p_effect, aout_instance_t *p_aout,
     if( !p_state)
     {
         free( height );
-        free( p_s16_buff );
         msg_Err(p_aout,"unable to initialize FFT transform");
         return -1;
     }
+    if( !window_init( FFT_BUFFER_SIZE, &p_data->wind_param, &wind_ctx ) )
+    {
+        fft_close( p_state );
+        free( height );
+        msg_Err(p_aout,"unable to initialize FFT window");
+        return -1;
+    }
     p_buffs = p_s16_buff;
     for ( i = 0 ; i < FFT_BUFFER_SIZE ; i++)
     {
@@ -178,6 +211,7 @@ int spectrum_Run(visual_effect_t * p_effect, aout_instance_t *p_aout,
             p_buffs = p_s16_buff;
 
     }
+    window_scale_in_place( p_buffer1, &wind_ctx );
     fft_perform( p_buffer1, p_output, p_state);
     for( i = 0; i< FFT_BUFFER_SIZE ; i++ )
         p_dest[i] = p_output[i] *  ( 2 ^ 16 ) / ( ( FFT_BUFFER_SIZE / 2 * 32768 ) ^ 2 );
@@ -325,20 +359,44 @@ int spectrum_Run(visual_effect_t * p_effect, aout_instance_t *p_aout,
         }
     }
 
+    window_close( &wind_ctx );
+
     fft_close( p_state );
 
-    free( p_s16_buff );
     free( height );
 
     return 0;
 }
 
+static void spectrum_Free( void *data )
+{
+    spectrum_data *p_data = data;
+
+    if( p_data != NULL )
+    {
+        free( p_data->peaks );
+        free( p_data->prev_heights );
+        free( p_data->p_prev_s16_buff );
+        free( p_data );
+    }
+}
+
 
 /*****************************************************************************
  * spectrometer_Run: derivative spectrum analysis
  *****************************************************************************/
-int spectrometer_Run(visual_effect_t * p_effect, aout_instance_t *p_aout,
-                 aout_buffer_t * p_buffer , picture_t * p_picture)
+typedef struct
+{
+    int *peaks;
+
+    unsigned i_prev_nb_samples;
+    int16_t *p_prev_s16_buff;
+
+    window_param wind_param;
+} spectrometer_data;
+
+static int spectrometer_Run(visual_effect_t * p_effect, vlc_object_t *p_aout,
+                            const block_t * p_buffer , picture_t * p_picture)
 {
 #define Y(R,G,B) ((uint8_t)( (R * .299) + (G * .587) + (B * .114) ))
 #define U(R,G,B) ((uint8_t)( (R * -.169) + (G * -.332) + (B * .500) + 128 ))
@@ -385,9 +443,10 @@ int spectrometer_Run(visual_effect_t * p_effect, aout_instance_t *p_aout,
     const double y_scale =  3.60673760222;  /* (log 256) */
 
     fft_state *p_state;                 /* internal FFT data */
+    DEFINE_WIND_CONTEXT( wind_ctx );    /* internal window data */
 
     int i , j , k;
-    int i_line;
+    int i_line = 0;
     int16_t p_dest[FFT_BUFFER_SIZE];      /* Adapted FFT result */
     int16_t p_buffer1[FFT_BUFFER_SIZE];   /* Buffer on which we perform
                                              the FFT (first channel) */
@@ -397,25 +456,56 @@ int spectrometer_Run(visual_effect_t * p_effect, aout_instance_t *p_aout,
     int16_t  *p_buffs;                    /* int16_t converted buffer */
     int16_t  *p_s16_buff;                /* int16_t converted buffer */
 
-    i_line = 0;
-
-    p_s16_buff = malloc( p_buffer->i_nb_samples * p_effect->i_nb_chans * sizeof(int16_t) );
-    if( !p_s16_buff )
+    if (!p_buffer->i_nb_samples) {
+        msg_Err(p_aout, "no samples yet");
         return -1;
+    }
 
-    p_buffs = p_s16_buff;
-    i_original     = config_GetInt ( p_aout, "spect-show-original" );
-    i_80_bands     = config_GetInt ( p_aout, "spect-80-bands" );
-    i_separ        = config_GetInt ( p_aout, "spect-separ" );
-    i_amp          = config_GetInt ( p_aout, "spect-amp" );
-    i_peak         = config_GetInt ( p_aout, "spect-show-peaks" );
-    i_show_base    = config_GetInt ( p_aout, "spect-show-base" );
-    i_show_bands   = config_GetInt ( p_aout, "spect-show-bands" );
-    i_rad          = config_GetInt ( p_aout, "spect-radius" );
-    i_sections     = config_GetInt ( p_aout, "spect-sections" );
-    i_extra_width  = config_GetInt ( p_aout, "spect-peak-width" );
-    i_peak_height  = config_GetInt ( p_aout, "spect-peak-height" );
-    color1         = config_GetInt ( p_aout, "spect-color" );
+    /* Create the data struct if needed */
+    spectrometer_data *p_data = p_effect->p_data;
+    if( !p_data )
+    {
+        p_data = malloc( sizeof(spectrometer_data) );
+        if( !p_data )
+            return -1;
+        p_data->peaks = calloc( 80, sizeof(int) );
+        if( !p_data->peaks )
+        {
+            free( p_data );
+            return -1;
+        }
+        p_data->i_prev_nb_samples = 0;
+        p_data->p_prev_s16_buff = NULL;
+        window_get_param( p_aout, &p_data->wind_param );
+        p_effect->p_data = (void*)p_data;
+    }
+    peaks = p_data->peaks;
+
+    /* Allocate the buffer only if the number of samples change */
+    if( p_buffer->i_nb_samples != p_data->i_prev_nb_samples )
+    {
+        free( p_data->p_prev_s16_buff );
+        p_data->p_prev_s16_buff = vlc_alloc( p_buffer->i_nb_samples *
+                                             p_effect->i_nb_chans,
+                                             sizeof(int16_t));
+        p_data->i_prev_nb_samples = p_buffer->i_nb_samples;
+        if( !p_data->p_prev_s16_buff )
+            return -1;
+    }
+    p_buffs = p_s16_buff = p_data->p_prev_s16_buff;
+
+    i_original     = var_InheritInteger( p_aout, "spect-show-original" );
+    i_80_bands     = var_InheritInteger( p_aout, "spect-80-bands" );
+    i_separ        = var_InheritInteger( p_aout, "spect-separ" );
+    i_amp          = var_InheritInteger( p_aout, "spect-amp" );
+    i_peak         = var_InheritInteger( p_aout, "spect-show-peaks" );
+    i_show_base    = var_InheritInteger( p_aout, "spect-show-base" );
+    i_show_bands   = var_InheritInteger( p_aout, "spect-show-bands" );
+    i_rad          = var_InheritInteger( p_aout, "spect-radius" );
+    i_sections     = var_InheritInteger( p_aout, "spect-sections" );
+    i_extra_width  = var_InheritInteger( p_aout, "spect-peak-width" );
+    i_peak_height  = var_InheritInteger( p_aout, "spect-peak-height" );
+    color1         = var_InheritInteger( p_aout, "spect-color" );
 
     if( i_80_bands != 0)
     {
@@ -428,32 +518,9 @@ int spectrometer_Run(visual_effect_t * p_effect, aout_instance_t *p_aout,
         i_nb_bands = 20;
     }
 
-    if( !p_effect->p_data )
-    {
-        p_effect->p_data=(void *)malloc( 80 * sizeof(int) );
-        if( !p_effect->p_data )
-        {
-            free( p_s16_buff );
-            return -1;
-        }
-        peaks = (int *)p_effect->p_data;
-        for( i = 0 ; i < i_nb_bands ; i++ )
-        {
-           peaks[i] = 0;
-        }
-    }
-    else
-    {
-        peaks =(int *)p_effect->p_data;
-    }
-
-    height = (int *)malloc( i_nb_bands * sizeof(int) );
+    height = vlc_alloc( i_nb_bands, sizeof(int) );
     if( !height)
-    {
-        free( p_effect->p_data );
-        free( p_s16_buff );
         return -1;
-    }
 
     /* Convert the buffer to int16_t  */
     /* Pasted from float32tos16.c */
@@ -472,8 +539,13 @@ int spectrometer_Run(visual_effect_t * p_effect, aout_instance_t *p_aout,
     {
         msg_Err(p_aout,"unable to initialize FFT transform");
         free( height );
-        free( p_effect->p_data );
-        free( p_s16_buff );
+        return -1;
+    }
+    if( !window_init( FFT_BUFFER_SIZE, &p_data->wind_param, &wind_ctx ) )
+    {
+        fft_close( p_state );
+        free( height );
+        msg_Err(p_aout,"unable to initialize FFT window");
         return -1;
     }
     p_buffs = p_s16_buff;
@@ -486,9 +558,13 @@ int spectrometer_Run(visual_effect_t * p_effect, aout_instance_t *p_aout,
         if( p_buffs >= &p_s16_buff[p_buffer->i_nb_samples * p_effect->i_nb_chans] )
             p_buffs = p_s16_buff;
     }
+    window_scale_in_place( p_buffer1, &wind_ctx );
     fft_perform( p_buffer1, p_output, p_state);
-    for(i= 0; i< FFT_BUFFER_SIZE ; i++ )
-        p_dest[i] = ( (int) sqrt( p_output [ i ] ) ) >> 8;
+    for(i = 0; i < FFT_BUFFER_SIZE; i++)
+    {
+        int sqrti = sqrt(p_output[i]);
+        p_dest[i] = sqrti >> 8;
+    }
 
     i_nb_bands *= i_sections;
 
@@ -504,9 +580,10 @@ int spectrometer_Run(visual_effect_t * p_effect, aout_instance_t *p_aout,
         y >>=7;/* remove some noise */
         if( y != 0)
         {
-            height[i] = (int)log(y)* y_scale;
-               if(height[i] > 150)
-                  height[i] = 150;
+            int logy = log(y);
+            height[i] = logy * y_scale;
+            if(height[i] > 150)
+                height[i] = 150;
         }
         else
         {
@@ -779,207 +856,230 @@ int spectrometer_Run(visual_effect_t * p_effect, aout_instance_t *p_aout,
         }
     }
 
+    window_close( &wind_ctx );
+
     fft_close( p_state );
 
-    free( p_s16_buff );
     free( height );
 
     return 0;
 }
 
+static void spectrometer_Free( void *data )
+{
+    spectrometer_data *p_data = data;
+
+    if( p_data != NULL )
+    {
+        free( p_data->peaks );
+        free( p_data->p_prev_s16_buff );
+        free( p_data );
+    }
+}
+
 
 /*****************************************************************************
  * scope_Run: scope effect
  *****************************************************************************/
-int scope_Run(visual_effect_t * p_effect, aout_instance_t *p_aout,
-              aout_buffer_t * p_buffer , picture_t * p_picture)
+static int scope_Run(visual_effect_t * p_effect, vlc_object_t *p_aout,
+                     const block_t * p_buffer , picture_t * p_picture)
 {
     VLC_UNUSED(p_aout);
+
     int i_index;
     float *p_sample ;
     uint8_t *ppp_area[2][3];
 
-
-        for( i_index = 0 ; i_index < 2 ; i_index++ )
+    for( i_index = 0 ; i_index < 2 ; i_index++ )
+    {
+        for( int j = 0 ; j < 3 ; j++ )
         {
-            int j;
-            for( j = 0 ; j < 3 ; j++ )
-            {
-                ppp_area[i_index][j] =
-                    p_picture->p[j].p_pixels + i_index * p_picture->p[j].i_lines
-                                / 2 * p_picture->p[j].i_pitch;
-            }
+            ppp_area[i_index][j] =
+                p_picture->p[j].p_pixels + (i_index * 2 + 1) * p_picture->p[j].i_lines
+                / 4 * p_picture->p[j].i_pitch;
         }
+    }
 
-        for( i_index = 0, p_sample = (float *)p_buffer->p_buffer;
-             i_index < __MIN( p_effect->i_width, p_buffer->i_nb_samples );
-             i_index++ )
-        {
-            uint8_t i_value;
+    for( i_index = 0, p_sample = (float *)p_buffer->p_buffer;
+            i_index < __MIN( p_effect->i_width, (int)p_buffer->i_nb_samples );
+            i_index++ )
+    {
+        int8_t i_value;
 
-            /* Left channel */
-            i_value =  p_sample[p_effect->i_idx_left] * 127;
-            *(ppp_area[0][0]
-               + p_picture->p[0].i_pitch * i_index / p_effect->i_width
-               + p_picture->p[0].i_lines * i_value / 512
-                   * p_picture->p[0].i_pitch) = 0xbf;
-            *(ppp_area[0][1]
+        /* Left channel */
+        i_value =  p_sample[p_effect->i_idx_left] * 127;
+        *(ppp_area[0][0]
+                + p_picture->p[0].i_pitch * i_index / p_effect->i_width
+                + p_picture->p[0].i_lines * i_value / 512
+                * p_picture->p[0].i_pitch) = 0xbf;
+        *(ppp_area[0][1]
                 + p_picture->p[1].i_pitch * i_index / p_effect->i_width
                 + p_picture->p[1].i_lines * i_value / 512
-                   * p_picture->p[1].i_pitch) = 0xff;
+                * p_picture->p[1].i_pitch) = 0xff;
 
 
-           /* Right channel */
-           i_value = p_sample[p_effect->i_idx_right] * 127;
-           *(ppp_area[1][0]
-              + p_picture->p[0].i_pitch * i_index / p_effect->i_width
-              + p_picture->p[0].i_lines * i_value / 512
-                 * p_picture->p[0].i_pitch) = 0x9f;
-           *(ppp_area[1][2]
-              + p_picture->p[2].i_pitch * i_index / p_effect->i_width
-              + p_picture->p[2].i_lines * i_value / 512
+        /* Right channel */
+        i_value = p_sample[p_effect->i_idx_right] * 127;
+        *(ppp_area[1][0]
+                + p_picture->p[0].i_pitch * i_index / p_effect->i_width
+                + p_picture->p[0].i_lines * i_value / 512
+                * p_picture->p[0].i_pitch) = 0x9f;
+        *(ppp_area[1][2]
+                + p_picture->p[2].i_pitch * i_index / p_effect->i_width
+                + p_picture->p[2].i_lines * i_value / 512
                 * p_picture->p[2].i_pitch) = 0xdd;
 
-           p_sample += p_effect->i_nb_chans;
-        }
-        return 0;
+        p_sample += p_effect->i_nb_chans;
+    }
+    return 0;
 }
 
 
 /*****************************************************************************
  * vuMeter_Run: vu meter effect
  *****************************************************************************/
-int vuMeter_Run(visual_effect_t * p_effect, aout_instance_t *p_aout,
-              aout_buffer_t * p_buffer , picture_t * p_picture)
+static int vuMeter_Run(visual_effect_t * p_effect, vlc_object_t *p_aout,
+                       const block_t * p_buffer , picture_t * p_picture)
 {
-        VLC_UNUSED(p_aout);
-        int i, j;
-        float i_value_l = 0;
-        float i_value_r = 0;
+    VLC_UNUSED(p_aout);
+    float i_value_l = 0;
+    float i_value_r = 0;
 
-        /* Compute the peack values */
-        for ( i = 0 ; i < p_buffer->i_nb_samples; i++ )
-        {
-                const float *p_sample = (float *)p_buffer->p_buffer;
-                float ch;
+    /* Compute the peak values */
+    for ( unsigned i = 0 ; i < p_buffer->i_nb_samples; i++ )
+    {
+        const float *p_sample = (float *)p_buffer->p_buffer;
+        float ch;
 
-                ch = p_sample[p_effect->i_idx_left] * 256;
-                if (ch > i_value_l)
-                        i_value_l = ch;
+        ch = p_sample[p_effect->i_idx_left] * 256;
+        if (ch > i_value_l)
+            i_value_l = ch;
 
-                ch = p_sample[p_effect->i_idx_right] * 256;
-                if (ch > i_value_r)
-                        i_value_r = ch;
+        ch = p_sample[p_effect->i_idx_right] * 256;
+        if (ch > i_value_r)
+            i_value_r = ch;
 
-                p_sample += p_effect->i_nb_chans;
-        }
+        p_sample += p_effect->i_nb_chans;
+    }
 
-        i_value_l = abs(i_value_l);
-        i_value_r = abs(i_value_r);
+    i_value_l = fabsf(i_value_l);
+    i_value_r = fabsf(i_value_r);
 
-        /* Stay under maximum value admited */
-        if ( i_value_l > 200 * M_PI_2 )
-                i_value_l = 200 * M_PI_2;
-        if ( i_value_r > 200 * M_PI_2 )
-                i_value_r = 200 * M_PI_2;
+    /* Stay under maximum value admitted */
+    if ( i_value_l > 200 * M_PI_2 )
+        i_value_l = 200 * M_PI_2;
+    if ( i_value_r > 200 * M_PI_2 )
+        i_value_r = 200 * M_PI_2;
 
-        float *i_value;
+    float *i_value;
 
-        if( !p_effect->p_data )
-        {
-                /* Allocate memory to save hand positions */
-                p_effect->p_data = (void *)malloc( 2 * sizeof(float) );
-                i_value = p_effect->p_data;
-                i_value[0] = i_value_l;
-                i_value[1] = i_value_r;
-        }
+    if( !p_effect->p_data )
+    {
+        /* Allocate memory to save hand positions */
+        p_effect->p_data = vlc_alloc( 2, sizeof(float) );
+        i_value = p_effect->p_data;
+        i_value[0] = i_value_l;
+        i_value[1] = i_value_r;
+    }
+    else
+    {
+        /* Make the hands go down slowly if the current values are slower
+           than the previous */
+        i_value = p_effect->p_data;
+
+        if ( i_value_l > i_value[0] - 6 )
+            i_value[0] = i_value_l;
         else
+            i_value[0] = i_value[0] - 6;
+
+        if ( i_value_r > i_value[1] - 6 )
+            i_value[1] = i_value_r;
+        else
+            i_value[1] = i_value[1] - 6;
+    }
+
+    int x, y;
+    float teta;
+    float teta_grad;
+
+    int start_x = p_effect->i_width / 2 - 120; /* i_width.min = 532 (visual.c) */
+
+    for ( int j = 0; j < 2; j++ )
+    {
+        /* Draw the two scales */
+        int k = 0;
+        teta_grad = GRAD_ANGLE_MIN;
+        for ( teta = -M_PI_4; teta <= M_PI_4; teta = teta + 0.003 )
         {
-                /* Make the hands go down slowly if the current values are slower
-                than the previous */
-                i_value = p_effect->p_data;
-
-                if ( i_value_l > i_value[0] - 6 )
-                        i_value[0] = i_value_l;
-                else
-                        i_value[0] = i_value[0] - 6;
-
-                if ( i_value_r > i_value[1] - 6 )
-                        i_value[1] = i_value_r;
-                else
-                        i_value[1] = i_value[1] - 6;
+            for ( unsigned i = 140; i <= 150; i++ )
+            {
+                y = i * cos(teta) + 20;
+                x = i * sin(teta) + start_x + 240 * j;
+                /* Compute the last color for the gradation */
+                if (teta >= teta_grad + GRAD_INCR && teta_grad <= GRAD_ANGLE_MAX)
+                {
+                    teta_grad = teta_grad + GRAD_INCR;
+                    k = k + 5;
+                }
+                *(p_picture->p[0].p_pixels +
+                        (p_picture->p[0].i_lines - y - 1 ) * p_picture->p[0].i_pitch
+                        + x ) = 0x45;
+                *(p_picture->p[1].p_pixels +
+                        (p_picture->p[1].i_lines - y / 2 - 1 ) * p_picture->p[1].i_pitch
+                        + x / 2 ) = 0x0;
+                *(p_picture->p[2].p_pixels +
+                        (p_picture->p[2].i_lines - y / 2 - 1 ) * p_picture->p[2].i_pitch
+                        + x / 2 ) = 0x4D + k;
+            }
         }
 
-        int x, y, k;
-        float teta;
-        float teta_grad;
-
-        for ( j = 0; j < 2; j++ )
+        /* Draw the two hands */
+        teta = (float)i_value[j] / 200 - M_PI_4;
+        for ( int i = 0; i <= 150; i++ )
         {
-                /* Draw the two scales */
-                k = 0;
-                teta_grad = GRAD_ANGLE_MIN;
-                for ( teta = -M_PI_4; teta <= M_PI_4; teta = teta + 0.003 )
-                {
-                        for ( i = 140; i <= 150; i++ )
-                        {
-                                y = i * cos(teta) + 20;
-                                x = i * sin(teta) + 150 + 240 * j;
-                                /* Compute the last color for the gradation */
-                                if (teta >= teta_grad + GRAD_INCR && teta_grad <= GRAD_ANGLE_MAX)
-                                {
-                                        teta_grad = teta_grad + GRAD_INCR;
-                                        k = k + 5;
-                                }
-                                *(p_picture->p[0].p_pixels +
-                                 (p_picture->p[0].i_lines - y - 1 ) * p_picture->p[0].i_pitch
-                                 + x ) = 0x45;
-                                *(p_picture->p[1].p_pixels +
-                                 (p_picture->p[1].i_lines - y / 2 - 1 ) * p_picture->p[1].i_pitch
-                                 + x / 2 ) = 0x0;
-                                *(p_picture->p[2].p_pixels +
-                                 (p_picture->p[2].i_lines - y / 2 - 1 ) * p_picture->p[2].i_pitch
-                                 + x / 2 ) = 0x4D + k;
-                        }
-                }
-
-                /* Draw the two hands */
-                teta = (float)i_value[j] / 200 - M_PI_4;
-                for ( i = 0; i <= 150; i++ )
-                {
-                        y = i * cos(teta) + 20;
-                        x = i * sin(teta) + 150 + 240 * j;
-                        *(p_picture->p[0].p_pixels +
-                         (p_picture->p[0].i_lines - y - 1 ) * p_picture->p[0].i_pitch
-                         + x ) = 0xAD;
-                        *(p_picture->p[1].p_pixels +
-                         (p_picture->p[1].i_lines - y / 2 - 1 ) * p_picture->p[1].i_pitch
-                         + x / 2 ) = 0xFC;
-                        *(p_picture->p[2].p_pixels +
-                         (p_picture->p[2].i_lines - y / 2 - 1 ) * p_picture->p[2].i_pitch
-                         + x / 2 ) = 0xAC;
-                }
-
-                /* Draw the hand bases */
-                for ( teta = -M_PI_2; teta <= M_PI_2 + 0.01; teta = teta + 0.003 )
-                {
-                        for ( i = 0; i < 10; i++ )
-                        {
-                                y = i * cos(teta) + 20;
-                                x = i * sin(teta) + 150 + 240 * j;
-                                *(p_picture->p[0].p_pixels +
-                                 (p_picture->p[0].i_lines - y - 1 ) * p_picture->p[0].i_pitch
-                                 + x ) = 0xFF;
-                                *(p_picture->p[1].p_pixels +
-                                 (p_picture->p[1].i_lines - y / 2 - 1 ) * p_picture->p[1].i_pitch
-                                 + x / 2 ) = 0x80;
-                                *(p_picture->p[2].p_pixels +
-                                 (p_picture->p[2].i_lines - y / 2 - 1 ) * p_picture->p[2].i_pitch
-                                 + x / 2 ) = 0x80;
-                        }
-                }
-
+            y = i * cos(teta) + 20;
+            x = i * sin(teta) + start_x + 240 * j;
+            *(p_picture->p[0].p_pixels +
+                    (p_picture->p[0].i_lines - y - 1 ) * p_picture->p[0].i_pitch
+                    + x ) = 0xAD;
+            *(p_picture->p[1].p_pixels +
+                    (p_picture->p[1].i_lines - y / 2 - 1 ) * p_picture->p[1].i_pitch
+                    + x / 2 ) = 0xFC;
+            *(p_picture->p[2].p_pixels +
+                    (p_picture->p[2].i_lines - y / 2 - 1 ) * p_picture->p[2].i_pitch
+                    + x / 2 ) = 0xAC;
         }
 
-        return 0;
+        /* Draw the hand bases */
+        for ( teta = -M_PI_2; teta <= M_PI_2 + 0.01; teta = teta + 0.003 )
+        {
+            for ( int i = 0; i < 10; i++ )
+            {
+                y = i * cos(teta) + 20;
+                x = i * sin(teta) + start_x + 240 * j;
+                *(p_picture->p[0].p_pixels +
+                        (p_picture->p[0].i_lines - y - 1 ) * p_picture->p[0].i_pitch
+                        + x ) = 0xFF;
+                *(p_picture->p[1].p_pixels +
+                        (p_picture->p[1].i_lines - y / 2 - 1 ) * p_picture->p[1].i_pitch
+                        + x / 2 ) = 0x80;
+                *(p_picture->p[2].p_pixels +
+                        (p_picture->p[2].i_lines - y / 2 - 1 ) * p_picture->p[2].i_pitch
+                        + x / 2 ) = 0x80;
+            }
+        }
+
+    }
+
+    return 0;
 }
+
+/* Table of effects */
+const struct visual_cb_t effectv[] = {
+    { "scope",        scope_Run,        dummy_Free        },
+    { "vuMeter",      vuMeter_Run,      dummy_Free        },
+    { "spectrum",     spectrum_Run,     spectrum_Free     },
+    { "spectrometer", spectrometer_Run, spectrometer_Free },
+    { "dummy",        dummy_Run,        dummy_Free        },
+};
+const unsigned effectc = sizeof (effectv) / sizeof (effectv[0]);
